@@ -1,6 +1,6 @@
 from chomsky import Whitespace, ParseException
 from runtime import Runtime, Continue, Skip, SuppressNewline, SuppressOneNewline
-from exceptions import PlywoodKeyError, this_line, BreakException, ContinueException
+from exceptions import PlywoodKeyError, this_line, BreakException, ContinueException, InvalidArguments
 from element import output_element
 
 
@@ -20,6 +20,15 @@ class PlywoodValue(object):
     def get_value(self, scope):
         return self
 
+    def call(self, states, scope, arguments, block):
+        if len(block.lines):
+            raise InvalidArguments('`str` does not support block argument')
+        if len(arguments.args) != 1 \
+            or len(arguments.kwargs):
+            raise InvalidArguments('`str` only accepts one conditional argument')
+        retval = self.python_value(scope) + arguments.args[0].python_value(scope)
+        return [Continue()], PlywoodWrapper(self.location, retval)
+
     def run(self, states, scope):
         if Continue() in states:
             return [Continue()], self.get_value(scope)
@@ -64,23 +73,35 @@ class PlywoodBlock(PlywoodValue):
         retval = ''
         try:
             for cmd in self.lines:
+                print("""=============== values.py at line {0} ===============
+cmd: {1!r}
+""".format(__import__('sys')._getframe().f_lineno - 2, cmd, ))
                 states, cmd_ret = cmd.run(states, scope)
+                # commands that *do* return a value, but do not want that value
+                # output, e.g. assignments.
                 if Skip() in states:
                     states.remove(Skip())
                     states.append(Continue())
-                    cmd_ret = None
                 else:
+                    print("""=============== values.py at line {0} ===============
+cmd_ret: {1!r}
+""".format(__import__('sys')._getframe().f_lineno - 2, cmd_ret, ))
+                    print("""=============== values.py at line {0} ===============
+cmd_ret.python_value(scope): {1!r}
+""".format(__import__('sys')._getframe().f_lineno - 2, cmd_ret.python_value(scope), ))
                     cmd_ret = cmd_ret.python_value(scope)
-                if cmd_ret is not None:
-                    retval += str(cmd_ret)
-                    suppress_newline = SuppressNewline() in states or SuppressOneNewline() in states
-                    if not self.inline and not suppress_newline:
-                        retval += scope['__separator']
-                    try:
-                        states.remove(SuppressOneNewline())
-                    except ValueError:
-                        pass
+                    if cmd_ret is not None:
+                        retval += str(cmd_ret)
+                        suppress_newline = SuppressNewline() in states or SuppressOneNewline() in states
+                        if not self.inline and not suppress_newline:
+                            retval += scope['__separator']
+                        try:
+                            states.remove(SuppressOneNewline())
+                        except ValueError:
+                            pass
         except (BreakException, ContinueException) as e:
+            # store the current output, because it is still relevant!
+            # plugins that support break and continue should return this string
             raise type(e)(retval)
 
         try:
@@ -95,6 +116,9 @@ class PlywoodVariable(PlywoodValue):
         self.name = name
         super(PlywoodVariable, self).__init__(location)
 
+    def run(self, states, scope):
+        return self.get_value(scope).run(states, scope)
+
     def get_name(self):
         return self.name
 
@@ -107,7 +131,8 @@ class PlywoodVariable(PlywoodValue):
             if not isinstance(retval, PlywoodValue):
                 retval = PlywoodWrapper(self.location, retval)
                 scope[self.name] = retval
-            retval.location = self.location
+            else:
+                retval.location = self.location
             return retval
         except KeyError:
             line_no, line = this_line(scope['__input'], self.location)
@@ -136,6 +161,15 @@ class PlywoodString(PlywoodValue):
         self.value = value
         super(PlywoodString, self).__init__(location)
 
+    def call(self, states, scope, arguments, block):
+        if len(block.lines):
+            raise InvalidArguments('`str` does not support block argument')
+        if len(arguments.args) != 1 \
+            or len(arguments.kwargs):
+            raise InvalidArguments('`str` only accepts one conditional argument')
+        retval = unicode(self.python_value(scope)) + unicode(arguments.args[0].python_value(scope))
+        return [Continue()], PlywoodWrapper(self.location, retval)
+
     @staticmethod
     def unindent(value, lang=False):
         indent = None
@@ -183,6 +217,7 @@ class PlywoodString(PlywoodValue):
         was_close_bracket = False
         inside = ''
         runtime = PlywoodEnv({'separator': ' '})
+        scope.push()
         context = scope.get('self', {})
         while index < len(original):
             c = original[index]
@@ -190,10 +225,8 @@ class PlywoodString(PlywoodValue):
                 consuming = False
                 was_close_bracket = False
                 inside = PlywoodString.unindent(inside)
-                old_input = scope.get('__input', '')
                 scope['__input'] = inside
                 retval += Plywood(inside).run(context, runtime).rstrip()
-                scope['__input'] = old_input
                 inside = ''
             elif consuming and c == '}':
                 was_close_bracket = True
@@ -214,7 +247,8 @@ class PlywoodString(PlywoodValue):
                 retval += c
             index += 1
         if was_open_bracket:
-            raise ParseException("Unclosed '{'")
+            raise ParseException("Unclosed '{{'")
+        scope.pop()
         return retval
 
     def __eq__(self, other):
@@ -254,6 +288,15 @@ class PlywoodNumber(PlywoodValue):
         self.value = value
         super(PlywoodNumber, self).__init__(location)
 
+    def call(self, states, scope, arguments, block):
+        if len(block.lines):
+            raise InvalidArguments('`str` does not support block argument')
+        if len(arguments.args) != 1 \
+            or len(arguments.kwargs):
+            raise InvalidArguments('`str` only accepts one conditional argument')
+        retval = self.python_value(scope) * arguments.args[0].python_value(scope)
+        return [Continue()], PlywoodWrapper(self.location, retval)
+
     def python_value(self, scope):
         return self.value
 
